###############   PSQF 6271 Longitudinal MLM Example 3a using R   #############

# Set width of output and  number of digits to be printed
options("width"=120, "digits"=8) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("nlme")) install.packages("nlme")
library(nlme) # To estimate MLMs with link functions and R cov structures

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in gls


################################################################################
#####               BEGIN EXAMPLE 3a DATA MANIUPLATION                     #####
################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22_PSQF6271/PSQF6271_Example3a/"
filename = "SAS_Chapter3a.sas7bdat"
setwd(dir=filesave)

# Import chapter 3 two-occasion multivariate data with labels
Example3a_wide = read_sas(data_file=paste0(filesave,filename)) 
# Convert to data frame as data frame without labels to use for analysis
Example3a_wide = as.data.frame(Example3a_wide)

# Stack into long format (one row per occasion per person)
Example3a = reshape(Example3a_wide, direction="long",
                    v.names="outcome", idvar="PersonID", timevar="occasion",
                    varying=c("outcome1","outcome2"), times=c(1,2))
# Center predictors for analysis
Example3a$time  = Example3a$occasion-1
Example3a$treat = Example3a$group-1
# Labels as comments only
# occasion: 1=pre-test, 2=post-test
# outcome: Learning Outcome
# time: 0=pre-test, 1=post-test
# treat: 0=control, 1=treatment

# Sort by person and occasion
Example3a = sort_asc(data=Example3a, PersonID,occasion)


################################################################################
#####                   BEGIN EXAMPLE 3a MODELS                            #####
################################################################################

# Open external text file to save results to
txtStart(file=paste0(filesave,"R_Example3a_Output.txt"))

print("Chapter 3a Example: Means by group and occasion for learning outcome")
describeBy(x=Example3a$outcome, digits=2,
           group=list(group=Example3a$group, occasion=Example3a$occasion))  

txtComment(" ") # insert blank space in output


print("Eq 3.1: Empty Between-Person model via R GLS (no correlation, =LM)")
empty_BP = gls(data=Example3a, method="REML", model=outcome~1, 
               correlation=NULL) # VC R matrix (no correlation)
print("Show results with -2LL, total original variance, and 95% CI")
summary(empty_BP); -2*logLik(empty_BP); summary(empty_BP)$sigma^2; confint(empty_BP, level=.95)

txtComment(" ") # insert blank space in output


print("Eq 3.2: Empty Within-Person model via R GLS (CS correlation)")
empty_WP = gls(data=Example3a, method="REML", model=outcome~1, 
               correlation=corCompSymm(form=~1|PersonID) )
print("Show results with -2LL, total original variance, and 95% CI")
summary(empty_WP); -2*logLik(empty_WP); summary(empty_WP)$sigma^2; confint(empty_WP, level=.95)  

print("Show RCOV and RCORR matrices") 
getVarCov(empty_WP); corMatrix(empty_WP$modelStruct$corStruct)[[5]] 
print("Show likelihood ratio test comparing empty means variance model fit")
anova(empty_WP, empty_BP)  # Order of "more", "fewer"

txtComment(" ") # insert blank space in output


print("Eq 3.7 (top): Between-Person Conditional (Predictor) Model via R GLS (no corr, =LM)")
print("Not using factor variables, manually dummy coding group and time")
cond_BP = gls(data=Example3a, method="REML", model=outcome~1+time+treat+time:treat,
              correlation=NULL)
print("Show results with -2LL, total leftover variance, and 95% CI")
-2*logLik(cond_BP); summary(cond_BP); summary(cond_BP)$sigma^2; confint(cond_BP, level=.95)   

print("Get linear combination estimates, add correct denominator DF and 95% CIs")
estimates_BP = glht(model=cond_BP, df=48, linfct=rbind(
  "Control Group Mean at Pre-Test"     =c(1,0,0,0), # Values= multipliers of ordered fixed effects
  "Control Group Mean at Post-Test"    =c(1,1,0,0),
  "Treatment Group Mean at Pre-Test"   =c(1,0,1,0),
  "Treatment Group Mean at Post-Test"  =c(1,1,1,1),
  "Time Slope for Control Group"       =c(0,1,0,0),
  "Time Slope for Treatment Group"     =c(0,1,0,1),
  "Group Slope at Pre-Test"            =c(0,0,1,0),
  "Group Slope at Post-Test"           =c(0,0,1,1),
  "Time*Group Interaction Slope"       =c(0,0,0,1)))
summary(estimates_BP, test=adjusted("none"))
confint(estimates_BP, level=.95, calpha=univariate_calpha()) 

print("Get and show joint test of model R2 using correct denominator DF")
modelF_BP=glht(model=cond_BP, df=48, linfct=c("time=0","treat=0","time:treat=0"))
BP=summary(modelF_BP, test=Ftest()); BP 
print("Get and show hidden results for F, dfnum, dfden, and p-value")
BP$test$fstat; BP$test$df; BP$df
pf(BP$test$fstat,df1=BP$test$df,df2=BP$df,lower.tail=FALSE)

txtComment(" ") # insert blank space in output


print("Eq 3.7 (bottom): Within-Person Conditional (Predictor) Model via R GLS (CS corr")
print("Not using factor variables, manually dummy coding group and time")
cond_WP = gls(data=Example3a, method="REML", model=outcome~1+time+treat+time:treat,
              correlation=corCompSymm(form=~1|PersonID))
print("Show results with -2LL, total leftover variance, and 95% CI")
-2*logLik(cond_WP); summary(cond_WP); summary(cond_WP)$sigma^2; confint(cond_WP, level=.95) 

print("Show RCOV and RCORR matrices") 
getVarCov(cond_WP); corMatrix(cond_WP$modelStruct$corStruct)[[5]] 

print("Get linear combination estimates, add correct denominator DF and 95% CIs")
estimates_WP = glht(model=cond_WP, df=48, linfct=rbind(
  "Control Group Mean at Pre-Test"     =c(1,0,0,0), # Values= multipliers of ordered fixed effects
  "Control Group Mean at Post-Test"    =c(1,1,0,0),
  "Treatment Group Mean at Pre-Test"   =c(1,0,1,0),
  "Treatment Group Mean at Post-Test"  =c(1,1,1,1),
  "Time Slope for Control Group"       =c(0,1,0,0),
  "Time Slope for Treatment Group"     =c(0,1,0,1),
  "Group Slope at Pre-Test"            =c(0,0,1,0),
  "Group Slope at Post-Test"           =c(0,0,1,1),
  "Time*Group Interaction Slope"       =c(0,0,0,1)))
summary(estimates_WP, test=adjusted("none"))
confint(estimates_WP, level=.95, calpha=univariate_calpha()) 

print("Get and show joint test of model R2 using correct denominator DF")
modelF_WP=glht(model=cond_WP, df=48, linfct=c("time=0","treat=0","time:treat=0"))
WP=summary(modelF_WP, test=Ftest()); WP 
print("Get and show hidden results for F, dfnum, dfden, and p-value")
WP$test$fstat; WP$test$df; WP$df
pf(WP$test$fstat,df1=WP$test$df,df2=WP$df,lower.tail=FALSE)

print("Show likelihood ratio test comparing conditional means variance model fit")
anova(cond_WP, cond_BP) # Order of "more", "fewer"

txtComment(" ") # insert blank space in output

##########################################################################################
# Close external text file of saved results
txtStop()
